#!/usr/bin/perl

#	FILE:    graph.pl
#	AUTHOR:  Matteo Redaelli (mredaelli@tiscalinet.it)
#	CREATED: 2000-01-21
#	MODIFIED:
#		2001-02-11 matteo
#               Added check about parameters... 
#	
#	AIM: search for a path in a graph!
#	You need a file that contains the matrix of weights
#	of the graph. For example:
#
#	0,1,2
#	1,0,0
#	0,2,1
#
#       The output is a list of 0 ore more lines: each line is a
#       solution such as "3:0 1 3" where the first number is the weight
#       of the path and the others are the list of nodes of the path.

use Getopt::Std;
use graph;
use Set;

sub print_graph
  {
    #	local $graph = @_[0];
    
    for (0..$rows) { 
      local $row = $_;
      for ( 0..$cols ) {
	print "$graph[$row][$_] ";
      }
      print "\n";
    }
  }

sub nextNode
  {
    local ($from, $to) = @_;
    
    return $graph[$from][$to];
  }

sub path
  {
    local ($to, $weight, @past) = @_;
    local $from = $past[$#past];
    
    #	print "past=@past, from=$from, to=$to, weight=$weight\n ";
    #	print "from: $from, past = @past\n";
    
    #	check if the nodes are directly connected
    
    local $w = &nextNode( $from, $to);
    local $new_w = $weight + $w;
    
    if ( $w > 0 and ( $max_weight == 0 || 
		      ($max_weight > 0 and $new_w <= $max_weight) ) ) 
      {
	local @new_past = @past;
	
	$new_past[$#new_past + 1] = $to;
	print "$new_w:@new_past\n";
	return 0 if ($opt_o);
      }
    
    #	the nodes aren't directly connected: I'll get each successor of $from
    #	that is not already visited (@past) and..
    
    for ( 0..$cols ) {
      local $new_to = $_;
      local $w = $graph[$from][$new_to];

      if ($w > 0 and 
	  ($to cmp $new_to) != 0 and 
	  (Set::element( $new_to, @past) == 1)) {
	$new_w = $weight + $w;
	local @new_past = @past;

	$new_past[$#new_past+1] = $new_to;

	# print "new_past=@new_past, new_to=$new_to";

	# check the weight
	if ( $max_weight == 0 or $new_weight <= $max_weight) {
	  $result = path( $to, $weight + $w, @new_past); 
	  return 0 if ( $result == 0 and $opt_o ); 
	}
      }
    }
    return 2;
    
  }


sub usage
  {
    print "*************************************************************\n";
    print "$0 \n";
    print "by Matteo Redaelli (mredaelli\@tiscalinet.it)\n";
    print "USAGE: perl $0 <options>\n";
    print "	-b number: begining node\n";
    print "	-e number: destination node\n";
    print "	-f string: file that contains graph infos \n";
    print "	-h this help \n";
    print "	-o only one solution \n";
    print "	-r number: number of rows\n";
    print "	-s character: separator in file (-f option)\n";
    print "	-w number: max weight (0 every path) \ n";
    print "*************************************************************\n";
    
    exit 1;
  }

#
# reading arguments
#
$opt_w = 0;
$opt_s = ',';

getopts('b:e:f:hor:s:w:') || usage();

if ( ! defined($opt_b) || 
     ! defined($opt_e) || 
     ! defined($opt_f) || 
     ! defined($opt_r) ||
     defined($opt_h)
   ) {
  &usage();
}

$file = $opt_f;
$nodes = $opt_r--;
$rows = $nodes;
$cols = $rows;
$separator = $opt_s;
$begin = $opt_b;
$end = $opt_e;
$max_weight = $opt_w;

#
# Now I read graph definition from file and I build $graph array
#

graph::load_graph($file, $separator, $nodes, @graph);

#
# Now I search the path you want
#
path( $end, 0, ($begin) );
